#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/10 10:53
# @Author  : Adyan
# @File    : proxy.py


import json
import logging
import random
import re
import time

import requests


class IpProxy:

    def __init__(self, count: int,url):
        self.count = count
        self.url = url

    def get_jingling(self):
        # url = f'http://ip.ipjldl.com/index.php/api/entry?method=proxyServer.generate_api_url&packid=0&fa=0&fetch_key=&groupid=0&qty={self.count}&time=100&pro=&city=&port=1&format=json&ss=5&css=&ipport=1&dt=1&specialTxt=3&specialJson=&usertype=2'
        r = json.loads(requests.get(self.url).text)
        ip_list = []
        try:
            if '登' in r.get('msg'):
                res = re.findall('(.*?)登', r.get("msg"))
                add_ip_url = f"http://www.jinglingdaili.com/Users-whiteIpAddNew.html?appid=13498&appkey=f3b53f02df1d5d94a2c16816b28846dc&type=dt&whiteip={res[0]}&index=1"
                res = requests.get(add_ip_url, timeout=10)
                r = json.loads(requests.get(self.url).text)

            logging.info(r)
            for item in r["data"]:
                ip_list.append("https://" + item['IP'])
        except:
            logging.info(r)
        if ip_list:
            return ip_list

    def get_taiyang(self):
        pass

    def get_zhima(self):
        pass

    def get_proxy(self):
        pass





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/9/2 13:34
# @Author  : Adyan
# @File    : monitor_rabbit.py
import json
import time
import traceback
import logging

from pika.exceptions import ConnectionClosedByBroker, AMQPChannelError, AMQPConnectionError


class MonitorRabbit:
    def __init__(
            self, rabbit_conn, redis_coon,
            redis_key=None, callback=None
    ):
        """
        :param rabbit_conn: rabbit链接
        :param redis_coon: redis链接
        :param redis_key: redis储存的键
        :param callback: 方法
        """
        self.rabbit_conn = rabbit_conn
        self.redis_coon = redis_coon
        self.redis_key = redis_key
        self._callback = callback

    def start_run(self):
        """
        监听队列
        :return:
        """
        while True:
            try:
                self.rabbit_conn.get_rabbit(self.callback)
            except ConnectionClosedByBroker:
                logging.info(f'error  [{ConnectionClosedByBroker}]')
                time.sleep(10)
                continue
            except AMQPChannelError:
                logging.info(f'error  [{AMQPChannelError}]')
                time.sleep(10)
                continue
            except AMQPConnectionError:
                # traceback.print_exc()
                logging.info(f'error  [{AMQPConnectionError}]')
                time.sleep(10)
                continue
            except:
                traceback.print_exc()
                logging.info(f'error  [{"unknow error"}]')
                time.sleep(10)
                continue

    def callback(self, channel, method, properties, body):
        """
        回调函数
        """
        try:
            req_body = body.decode('utf-8')
            logging.info(req_body)
            mes = {'result': json.loads(req_body)}
            if self._callback:
                self._callback.shop_start(json.dumps(mes))
            else:
                self.redis_coon.lpush(f'{self.redis_key}:start_urls', json.dumps(mes, ensure_ascii=False))
        except Exception as e:
            print(e)
        finally:
            channel.basic_ack(delivery_tag=method.delivery_tag)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/8/2 17:43
# @Author  : Adyan
# @File    : Redis_conn.py
import json

import redis

from datetime import datetime


class ReidsClient(object):

    def __init__(self, **kwargs):
        """
        :param kwargs: {
            "HOST": "119.29.9.92",
            "PORT": 6379,
            "DB": 11,
            "NAME":'proxy'
            }
        """
        host = kwargs.get('HOST', 'localhost')
        port = kwargs.get('PORT', 6379)
        password = kwargs.get('PAW', None)
        db = kwargs.get('DB', 0)
        NAME = kwargs.get('NAME', None)
        if password:
            self.redis_conn = redis.Redis(host=host, port=port, password=password)
        else:
            self.redis_conn = redis.Redis(host=host, port=port, db=db)
        self.NAME = NAME

    def get(self, count):
        lst = self.redis_conn.lrange(self.NAME, 0, count - 1)
        self.redis_conn.ltrim(self.NAME, count, -1)
        return lst

    def put(self, param):
        self.redis_conn.rpush(self.NAME, param)

    @property
    def queue_len(self):
        return self.redis_conn.llen(self.NAME)

    def prox(self):
        ip_list = []
        data_list = self.redis_conn.hgetall("proxy")
        for ip_item in data_list:
            proxy_expire_time = int(data_list[ip_item].decode())
            now_time = int(datetime.now().timestamp())
            time_dif = proxy_expire_time - now_time
            if time_dif < 10:
                print(ip_item.decode(), "过期 删除这个代理")
                self.redis_conn.hdel('proxy', ip_item)
            else:
                ip_list.append(ip_item.decode())
        return ip_list




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/25 16:33
# @Author  : Adyan
# @File    : my_middleware.py


import random
import time
import requests
import logging

# from twisted.internet import defer, reactor
from twisted.internet import defer
from twisted.internet.error import ConnectionRefusedError

from scrapy import signals
from scrapy.http import TextResponse
from scrapy.core.downloader.handlers.http11 import TunnelError, TimeoutError
from gerapy_pyppeteer.downloadermiddlewares import reactor

from .proxy import IpProxy


class Proxy(object):

    def __init__(self, settings):
        self.settings = settings
        self.ip_list = []
        self.ip_data = 0
        self.get_ip = IpProxy(10)

    @classmethod
    def from_crawler(cls, crawler):
        return cls(crawler.settings)

    def process_response(self, request, response, spider):
        """
        处理响应
        :param request:
        :param response:
        :param spider:
        :return:
        """
        try:
            if spider.proxy:
                start_time = request.meta.get('_start_time', time.time())
                logging.info(
                    f'【代理{request.meta["proxy"][8:]}消耗时间】 {request.url} {time.time() - start_time}'
                )
                del request.meta["proxy"]
        except:
            pass
        return response

    def process_request(self, request, spider):
        """
        处理请求
        :param request:
        :param spider:
        :return:
        """
        request.meta.update(
            {
                '_start_time': time.time()
            }
        )
        try:
            proxy_switch = spider.proxy
        except:
            proxy_switch = False

        if proxy_switch:
            # if 'rate.1688.com' in request.url:
            for i in range(3):
                if self.ip_data < time.time():
                    self.ip_list.clear()
                try:
                    if len(self.ip_list) < 2:
                        self.ip_list = self.get_ip.get_jingling()
                        self.ip_data = time.time() + 300
                        time.sleep(3)
                    else:
                        break
                except:
                    self.ip_list = self.get_ip.get_jingling()
                    self.ip_data = time.time() + 300
                    time.sleep(3)
            request.meta['download_timeout'] = 5

            if self.ip_list:
                self.ip_raw = random.choice(self.ip_list)
                self.ip_list.remove(self.ip_raw)
                request.meta["proxy"] = self.ip_raw
            else:
                logging.info('代理列表为空')

    def process_exception(self, request, exception, spider):
        """
        过滤代理错误
        :param request:
        :param exception:
        :param spider:
        :return:
        """
        if isinstance(exception, (TunnelError, TimeoutError, ConnectionRefusedError)):
            return request


class Request(object):
    # Not all methods need to be defined. If a method is not defined,
    # scrapy acts as if the downloader middleware does not modify the
    # passed objects.

    @classmethod
    def from_crawler(cls, crawler):
        # This method is used by Scrapy to create your spiders.
        s = cls()
        crawler.signals.connect(s.spider_opened, signal=signals.spider_opened)
        return s

    @defer.inlineCallbacks
    def process_request(self, request, spider):
        container = []
        out = defer.Deferred()
        reactor.callInThread(self._get_res, request, container, out)
        yield out
        if len(container) > 0:
            defer.returnValue(container[0])

    def _get_res(self, request, container, out):
        url = request.url
        r = requests.get(url, headers=request.meta.get("headers"))
        r.encoding = request.encoding
        text = r.content

        # response = TextResponse(url=r.url, status=r.status_code, body=r.text, request=request)
        response = TextResponse(url=r.url, encoding="gbk", body=text, request=request)
        container.append(response)
        reactor.callFromThread(out.callback, response)
        # except Exception as e:
        #     err = str(type(e)) + ' ' + str(e)
        #     reactor.callFromThread(out.errback, ValueError(err))

    def process_response(self, request, response, spider):
        # Called with the response returned from the downloader.

        # Must either;
        # - return a Response object
        # - return a Request object
        # - or raise IgnoreRequest
        return response

    def process_exception(self, request, exception, spider):
        # Called when a download handler or a process_request()
        # (from other downloader middleware) raises an exception.

        # Must either:
        # - return None: continue processing this exception
        # - return a Response object: stops process_exception() chain
        # - return a Request object: stops process_exception() chain
        pass

    def spider_opened(self, spider):
        spider.logger.info('Spider opened: %s' % spider.name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/11/26 14:20
# @Author  : Adyan
# @File    : setup.py


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zhanlan2",
    version="0.1.7",
    author="Adyan",
    author_email="228923910@qq.com",
    description="Special package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/liujiang9/Utils",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)
